<?php

namespace Drupal\notify_to_slack\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\notify_to_slack\SlackMessageInterface;

/**
 * Defines Slack Message Config Entity.
 *
 * @ConfigEntityType(
 *   id = "slack_message",
 *   label = @Translation("Slack Message"),
 *   handlers = {
 *     "list_builder" = "Drupal\notify_to_slack\Controller\SlackMessageListBuilder",
 *     "form" = {
 *        "add" = "Drupal\notify_to_slack\Form\SlackMessageAddForm",
 *        "edit" = "Drupal\notify_to_slack\Form\SlackMessageEditForm",
 *        "delete" = "Drupal\notify_to_slack\Form\SlackMessageDeleteForm"
 *     },
 *   },
 *   config_prefix = "slack_message",
 *   admin_permission= "administer site configuration",
 *   entity_keys = {
 *    "id" = "id",
 *    "label" = "label"
 *   },
 *   config_export = {
 *    "id",
 *    "label",
 *   "entity_type",
 *   "bundle",
 *   "slack_message_set",
 *   },
 *   links = {
 *     "edit-form" = "/admin/config/slack-notification/message/{slack_message}",
 *     "delete-form" = "/admin/config/slack-notification/message/{slack_message}/delete",
 *   }
 * )
 */
class SlackMessage extends ConfigEntityBase implements SlackMessageInterface {

  /**
   * The Entity ID.
   *
   * @var string
   */
  public $id;

  /**
   * The Entity label.
   *
   * @var string
   */
  public $label;

  /**
   * The Entity Type for which SlackMessage entity is created.
   *
   * @var string
   */
  protected $entity_type;

  /**
   * Bundle of Entity for which SlackMessage entity is created.
   *
   * @var string
   */
  protected $bundle;

  /**
   * Slack Message Set i.e information on entity operation (e.g. update).
   *
   * @var array
   */
  public $slack_message_set = [];

  /**
   * {@inheritdoc}
   */
  public function getAttachedEntityType() {
    return $this->get('entity_type');
  }

  /**
   * {@inheritdoc}
   */
  public function setSlackMessageSet(FormStateInterface $form_state) {
    $slack_message_set = [];
    $operation_types = ['insert', 'update', 'delete'];
    foreach ($operation_types as $operation) {
      $slack_message_set[$operation] = [
        'state' => $form_state->getValue($operation . '_state'),
        'message' => $form_state->getValue($operation . '_message'),
      ];
    }
    $this->set('slack_message_set', $slack_message_set);
    return $this;
  }

  /**
   * Get Entity type from SlackMessage ID.
   *
   * @param \Drupal\notify_to_slack\Entity\SlackMessage $slackMessage
   *   Object of SlackMessage Entity.
   *
   * @return mixed
   *   Name of the Entity Type.
   */
  public function getAttachedEntityTypeFromSlackMessageId(SlackMessage $slackMessage) {
    $slack_message_id = $slackMessage->id();
    $slack_message_id_array = explode('__', $slack_message_id);
    return $slack_message_id_array[0];
  }

}
