<?php

namespace Drupal\notify_to_slack\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Render\RendererInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form that configures notify_to_slack settings.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;
  /**
   * The Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a \Drupal\notify_to_slack\Form\SettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The Entity Type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   Module handler.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   Renderer.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entityTypeManager, ModuleHandlerInterface $moduleHandler, RendererInterface $renderer) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entityTypeManager;
    $this->moduleHandler = $moduleHandler;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('renderer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'notify_slack_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'notify_to_slack.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config('notify_to_slack.settings');
    $rendered_token_tree = '';
    $token_tree = [
      '#theme' => 'token_tree_link',
      '#token_types' => array_keys($this->getContentEntityTypeList()),
      '#global_types' => FALSE,
    ];
    $rendered_token_tree = $this->renderer->render($token_tree);
    $form['url'] = [
      '#type' => 'url',
      '#title' => $this->t('Incoming Webhook URL'),
      '#default_value' => $config->get('url'),
      '#description' => $this->t('Enter incoming webhook url from slack account.'),
      '#required' => TRUE,
    ];

    $form['channel'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Slack Channel'),
      '#default_value' => $config->get('channel')?: 'general',
      '#description' => $this->t('Enter slack channel for notification.'),
      '#required' => TRUE,
    ];

    $form['bot_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Bot Name'),
      '#default_value' => $config->get('bot_name')?: 'Drupal Slack Bot',
      '#description' => $this->t('Enter display name of the notifier.'),
      '#required' => TRUE,
    ];

    $form['entity_type'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select Entity Type for which you want to send notifications.'),
      '#description' => $this->t('Select Entity Type for which you want to send notifications.'),
      '#default_value' => $config->get('entity_type')?: [],
      '#options' => $this->getContentEntityTypeList(),
      '#required' => TRUE,
    ];
    $operations = [
      'insert' => 'Insert',
      'update' => 'Update',
      'delete' => 'Delete',
    ];
    $form['operations'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select the notification operation.'),
      '#description' => $this->t('Select the operation for which slack message needs to be send.'),
      '#default_value' => $config->get('operations')?: [],
      '#options' => $operations,
      '#required' => TRUE,
    ];

    $form['default_message'] = [
      '#type' => 'details',
      '#title' => $this->t('Default messages'),
      '#open' => TRUE,
    ];
    foreach ($operations as $operation => $operation_label) {
      $form['default_message'][$operation . '_message'] = [
        '#type' => 'textarea',
        '#title' => $this->t('@operation_label Message', ['@operation_label' => $operation_label]),
        '#default_value' => $config->get($operation . '_message')?: '',
        '#required' => TRUE,
      ];
      $form['default_message'][$operation . '_message']['#description'] = $this->t('@token', ['@token' => $rendered_token_tree]);

    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('notify_to_slack.settings')
      ->set('url', $form_state->getValue('url'))
      ->set('channel', $form_state->getValue('channel'))
      ->set('bot_name', $form_state->getValue('bot_name'))
      ->set('entity_type', $form_state->getValue('entity_type'))
      ->set('operations', $form_state->getValue('operations'))
      ->set('insert_message', $form_state->getValue('insert_message'))
      ->set('update_message', $form_state->getValue('update_message'))
      ->set('delete_message', $form_state->getValue('delete_message'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Get the List of content entity types.
   *
   * @return array
   *   List of Content Entity Types.
   */
  public function getContentEntityTypeList() {
    $contentEntityTypeList = [];

    foreach ($this->entityTypeManager->getDefinitions() as $key => $entityType) {
      // Check weather entity type is ContentEntityType.
      if ($entityType instanceof ContentEntityTypeInterface) {
        $contentEntityTypeList[$key] = $entityType->getLabel();
      }
    }
    return $contentEntityTypeList;
  }

}
