<?php

namespace Drupal\notify_to_slack\Form;

use Drupal\Core\Form\FormStateInterface;

/**
 * SlackMessage entity edit form.
 */
class SlackMessageAddForm extends SlackMessageFormBase {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    $entity_type = $this->getContentEntityTypeList();

    $form['entity_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity Type'),
      '#options' => $this->getContentEntityTypeList(),
      '#default_value' => key($entity_type),
      '#required' => TRUE,
      '#ajax' => [
        'callback' => '::updateBundle',
        'wrapper' => 'edit-bundle-wrapper',
      ],
    ];

    $form['bundle'] = [
      '#type' => 'select',
      '#title' => $this->t('Bundle'),
      '#options' => $this->getEntityBundleList(key($entity_type)),
      '#required' => TRUE,
      '#empty_option' => $this->t('- Any -'),
      '#empty_value' => 'all_bundle',
      '#prefix' => '<div id="edit-bundle-wrapper">',
      '#suffix' => '</div>',
      '#validated' => TRUE,
    ];

    return $form;
  }

  /**
   * Handles switching the configuration type selector.
   */
  public function updateBundle($form, FormStateInterface $form_state) {
    $form['bundle']['#default_value'] = 'all_bundle';
    $form['bundle']['#options'] = $this->getEntityBundleList($form_state->getValue('entity_type'));
    return $form['bundle'];

  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    parent::validateForm($form, $form_state);

    $entityType = $form_state->getValue('entity_type');
    $bundle = $form_state->getValue('bundle');
    $slackMessageId = $entityType . '__' . $bundle;
    $slackMessageEntity = $this->entityTypeManager
      ->getStorage('slack_message')
      ->load($slackMessageId);

    if (!empty($slackMessageEntity)) {
      $form_state->setErrorByName('bundle', $this->t('Slack Message Set is already preset for the requested bundle @bundle', ['@bundle' => $bundle]));
    }
    else {
      $form_state->setValue('id', $slackMessageId);
      $form_state->setValue('label', $entityType . ' ' . $bundle);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $this->t(
      'Slack Message set @label was created.', ['@label' => $this->entity->label()]);
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    $actions['submit']['#value'] = $this->t('Create Slack Message set');
    return $actions;
  }

  /**
   * Get the List of content entity types.
   *
   * @return array
   *   List of Content Entity Types.
   */
  public function getContentEntityTypeList() {
    $contentEntityTypeList = [];
    $entity_types = $this->config->get('entity_type');
    foreach ($entity_types as $entity_type) {
      if (!empty($entity_type)) {
        $contentEntityTypeList[$entity_type] = $this->entityTypeManager
          ->getDefinition($entity_type)
          ->getLabel();
      }
    }
    return $contentEntityTypeList;
  }

  /**
   * Get the List of bundles for an entity.
   *
   * @param string $entityType
   *   Name of the entity type.
   *
   * @return array
   *   List of bundles for an entity.
   */
  public function getEntityBundleList($entityType) {
    $bundleList = ['all_bundle' => $this->t('- Any -')];
    if (empty($entityType)) {
      return $bundleList;
    }
    $bundles = $this->entityTypeBundleInfo->getBundleInfo($entityType);

    foreach ($bundles as $key => $bundle) {
      $bundleList[$key] = isset($bundle['label']) ? $bundle['label'] : $key;
    }

    return $bundleList;

  }

}
