<?php

namespace Drupal\notify_to_slack;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Utility\Token;
use GuzzleHttp\ClientInterface;
use Psr\Log\LoggerInterface;

/**
 * Slack Client Service to send notification on the Slack App.
 */
class SlackClient {

  /**
   * Config Object for 'notify_to_slack.settings'.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * HTTP Client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $client;

  /**
   * Incoming webhook url.
   *
   * @var string
   */
  protected $url;

  /**
   * The Messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Token manager.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * The SlackMessage.
   *
   * @var \Drupal\notify_to_slack\Entity\SlackMessage
   */
  protected $slackMessage;

  use StringTranslationTrait;

  /**
   * SlackClient constructor.
   *
   * @param \GuzzleHttp\ClientInterface $client
   *   The HTTP Client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory for retrieving required config objects.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger instance.
   * @param \Drupal\Core\Utility\Token $token
   *   The Token manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(ClientInterface $client, ConfigFactoryInterface $configFactory, MessengerInterface $messenger, LoggerInterface $logger, Token $token, EntityTypeManagerInterface $entityTypeManager) {
    $this->client = $client;
    $this->config = $configFactory->get('notify_to_slack.settings');
    $this->url = $this->config->get('url');
    $this->messenger = $messenger;
    $this->logger = $logger;
    $this->token = $token;
    $this->slackMessage = $entityTypeManager->getStorage('slack_message');
  }

  /**
   * Prepare the notification message.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The Object of type entity.
   * @param string $entityType
   *   The Entity Type.
   * @param string $action
   *   Type of the operation.
   *
   * @return mixed|bool
   *   The Message Object.
   */
  public function prepareMessage(EntityInterface $entity, $entityType, $action) {

    $message = NULL;
    $bundle = $entity->bundle();

    $slackMessageEntity = $this->loadSlackMessageEntity($entityType, $bundle);

    if (!empty($slackMessageEntity)) {
      $operations = $slackMessageEntity->get('slack_message_set');
      $state = $operations[$action]['state'];
      if (empty($state)) {
        // Sending message is disabled.
        return FALSE;
      }
      $message = $operations[$action]['message'];
    }

    if (empty($message)) {
      $message = $this->getDefaultSlackMessage($action);
    }

    $message = $this->token->replace($message, [$entityType => $entity]);
    $attachments = [];
    $message = Json::encode(
      [
        'text' => $message,
        'username' => $this->config->get('bot_name'),
        'channel' => "{$this->config->get('channel')}",
        "attachments" => $attachments,
      ]
    );
    return $message;

  }

  /**
   * Send Notification to Slack.
   *
   * @param mixed $message
   *   Message Object.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function sendNotification($message) {
    try {

      $this->client->request(
        'POST',
        $this->url,
        [
          'form_params' => [
            'payload' => $message,
          ],
        ]
      );
    }
    catch (GuzzleException $exception) {
      $this->displayAndLogExceptionMessage($exception);
    }
  }

  /**
   * Display and Log the exceptions.
   *
   * @param GuzzleException $exception
   *   The Exception object.
   */
  public function displayAndLogExceptionMessage(GuzzleException $exception) {
    $this->messenger->addMessage(
      $this->t('We are not able to send the information on slack. Please check logs for more info.'),
      'error'
    );
    $this->logger->error("An error occurred while processing the request on the webhook url. Message received is: " . $exception->getMessage());
  }

  /**
   * Get default Message for each operation.
   *
   * @param string $action
   *   Type of the operation.
   *
   * @return string
   *   Default Message.
   */
  protected function getDefaultSlackMessage($action) {
    return $this->config->get($action . '_message');
  }

  /**
   * Load SlackMessage entity.
   *
   * @param string $entityType
   *   The entity type.
   * @param string $bundle
   *   The bundle name.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null|static
   *   SlackMessage entity.
   */
  public function loadSlackMessageEntity($entityType, $bundle) {
    $default_bundle = 'all_bundle';
    $slack_message_id = $entityType . '__' . $bundle;
    $slack_message_entity = $this->slackMessage->load($slack_message_id);
    if (empty($slack_message_entity)) {
      $slack_message_id = $entityType . '__' . $default_bundle;
      $slack_message_entity = $this->slackMessage->load($slack_message_id);
    }
    return $slack_message_entity;
  }

}
